include "Vram.inc"
include "VVram.inc"
include "Chars.inc"

ext VVramFront_, VVramBack_
ext SpritePattern

VramTop equ Vram+VramRowSize*1
MMR_Vram equ $fd80+(Vram shr 12)

dseg
CharPattern: public CharPattern
    defs PatternSize*Char_End

BackupBack:
    defs VVramWidth*VVramHeight
BackupFront:
    defs VVramWidth*VVramHeight

zseg
yCount:
    defb 0
xCount:
    defb 0
vramPage:
    defb 0
pVram:
    defw 0


; void InitPalette();
cseg
InitPalette:
    ldx #$fd32
    do
        lsra
        if cs
            ldb #$0f
        else
            clrb
        endif
        stb ,x+
        cmpx #$fd32+3
    while ne | wend
rts
InitPalette_: public InitPalette_
    pshs a,b,x,y
        ldx #Palette
        ldb #64
        do
            pshs b
                ldd ,x++
                std $fd30
                lda ,x+ | sta $fd32
                lda ,x+ | sta $fd33
                lda ,x+ | sta $fd34
            puls b
            decb
        while ne | wend
    puls a,b,x,y
rts


; void ClearScreen();
cseg
ClearScreen_: public ClearScreen_
    pshs a,b,x,y
        clrb
        do
            stb $f430
            lda #$10
            do
                sta MMR_Vram
                ldx #Vram
                do
                    clr ,x+
                                ; pshs b | ldb #$ff | stb ,x+ | puls b
                    cmpx #Vram+$1000
                while ne | wend
                inca
                cmpa #$1c
            while cs | wend
            addb #$20
            cmpb #$40
        while ne | wend
        clr $f430

        ldx #BackupBack
        ldy #VVramWidth*VVramHeight*2
        do
            clr ,x+
            leay -1,y
        while ne | wend
    puls a,b,x,y
rts


cseg
Put: ;(a:page, x, y)
    do
        sta MMR_Vram
        inca
        sta MMR_Vram+1
        adda #3
        pshs a
            ldb #CharHeight
            do
                lda ,y+ | sta ,x
                leax VramWidth,x
                decb
            while ne | wend
            leax -VramWidth*CharHeight,x
        puls a
        cmpa #$1c
    while cs | wend
rts


; word Put(word vram, byte c);
cseg
Put_: public Put_
    pshs x,y
        lda #PatternSize
        mul
        addd #CharPattern
        tfr d,y

        lda #$10
        bsr Put
    puls x,y
    tfr x,d
    addd #1
rts


; void VVramToVram();
cseg
VVramToVram:
    lda #VVramHeight
    sta <yCount
    do
        lda #VVramWidth
        sta <xCount
        do 
            ldb ,x+
            cmpb ,y
            if ne
                stb ,y
                pshs x,y
                    lda #PatternSize
                    mul
                    addd #CharPattern
                    tfr d,y
                    ldx <pVram
                    lda <vramPage
                    bsr Put
                puls x,y
            endif
            leay 1,y
            pshs x
                ldx <pVram
                leax 1,x
                stx <pVram
            puls x
            dec <xCount
        while ne | wend
        pshs x
            ldx <pVram
            leax VramRowSize-VVramWidth,x
            stx <pVram
        puls x
        dec <yCount
    while ne | wend
rts
VVramToVram_: public VVramToVram_
    pshs a,b,x,y
        ldx #VramTop | stx <pVram
        ldx #VVramBack_
        ldy #BackupBack
        lda #$12 | sta <vramPage
        bsr VVramToVram
        
        ldx #VramTop | stx <pVram
        ldx #VVramFront_
        ldy #BackupFront
        lda #$10 | sta <vramPage
        bsr VVramToVram
    puls a,b,x,y
rts


cseg
Palette: ; 64
defw 0
defb 0,0,0
defw 4
defb 15,0,0
defw 8
defb 15,0,0
defw 12
defb 15,0,0
defw 64
defb 0,15,0
defw 68
defb 15,15,0
defw 72
defb 15,0,0
defw 76
defb 15,0,0
defw 128
defb 0,15,0
defw 132
defb 0,15,0
defw 136
defb 15,15,0
defw 140
defb 15,15,0
defw 192
defb 0,15,0
defw 196
defb 0,15,0
defw 200
defb 15,15,0
defw 204
defb 15,15,0
defw 1024
defb 0,0,15
defw 1028
defb 15,0,15
defw 1032
defb 15,0,0
defw 1036
defb 15,0,0
defw 1088
defb 0,15,15
defw 1092
defb 15,15,15
defw 1096
defb 15,0,0
defw 1100
defb 15,0,0
defw 1152
defb 0,15,0
defw 1156
defb 0,15,0
defw 1160
defb 15,15,0
defw 1164
defb 15,15,0
defw 1216
defb 0,15,0
defw 1220
defb 0,15,0
defw 1224
defb 15,15,0
defw 1228
defb 15,15,0
defw 2048
defb 0,0,15
defw 2052
defb 0,0,15
defw 2056
defb 15,0,15
defw 2060
defb 15,0,15
defw 2112
defb 0,0,15
defw 2116
defb 0,0,15
defw 2120
defb 15,0,15
defw 2124
defb 15,0,15
defw 2176
defb 0,15,15
defw 2180
defb 0,15,15
defw 2184
defb 15,15,15
defw 2188
defb 15,15,15
defw 2240
defb 0,15,15
defw 2244
defb 0,15,15
defw 2248
defb 15,15,15
defw 2252
defb 15,15,15
defw 3072
defb 0,0,15
defw 3076
defb 0,0,15
defw 3080
defb 15,0,15
defw 3084
defb 15,0,15
defw 3136
defb 0,0,15
defw 3140
defb 0,0,15
defw 3144
defb 15,0,15
defw 3148
defb 15,0,15
defw 3200
defb 0,15,15
defw 3204
defb 0,15,15
defw 3208
defb 15,15,15
defw 3212
defb 15,15,15
defw 3264
defb 0,15,15
defw 3268
defb 0,15,15
defw 3272
defb 15,15,15
defw 3276
defb 15,15,15
